@@ -356,6 +356,19 @@ def getcallbacktable():
     if not callbacktable: boot()
     return callbacktable.data
 
+def getretable():  
+    """ save command -> plugin list to db backend. """
+    global retable
+    if not retable: boot()
+    return retable.data
+
+def getshorttable():
+    """ save command -> plugin list to db backend. """
+    global shorttable
+    if not shorttable: boot()
+    return shorttable.data
+
+
 ## plugin list related commands
 
 def savepluginlist(modname=None):
