@@ -254,7 +254,7 @@ class Config(LazyDict):
             self._comments["loadlist"] = "# - loadlist .. not used yet."
             self.setdefault('loadlist', [])
             self._comments["quitmsg"] = "# - message to send on quit"
-            self.setdefault('quitmsg', "http://fbfbot.googlecode.com")
+            self.setdefault('quitmsg', "http://github.com/feedbackflow/fbfbot")
             self._comments["dotchars"] = "# - characters to used as seperator."
             self.setdefault('dotchars',  ", ")
             self._comments["floodallow"] = "# - whether the bot is allowed to flood."
