@@ -320,10 +320,13 @@ class EventBase(LazyDict):
 
     def getcc(self):
         if self.chan: cc = self.chan.data.cc
-        else: cc = "!;"
-        if not ';' in cc: cc += ";"
-        cfg = getmainconfig()
-        if cfg.globalcc: cc += cfg.globalcc
+        else: cc = ""
+        if not cc:
+            cfg = getmainconfig()
+            if cfg.globalcc and not cfg.globalcc in cc: cc += cfg.globalcc
+        if not cc: cc = "!;"
+        if not ";" in cc: cc += ";"
+        logging.debug("cc is %s" % cc)
         return cc
 
     def blocked(self):
