@@ -137,7 +137,7 @@ def log_write(m):
     })
     global loggers
     try: loggers[logname].info(line.strip())
-    except KeyError: logging.error("no logger available for channel %s" % logname)
+    except KeyError: logging.debug("no logger available for channel %s" % logname)
     except Exception as ex: handle_exception()
 
 backends['log'] = log_write
