@@ -42,6 +42,8 @@ from fbf.lib.datadir import getdatadir
 from fbf.lib.channelbase import ChannelBase
 from fbf.imports import getfeedparser, getjson
 from fbf.lib.cache import get, set, delete
+from fbf.lib.runner import threadrunner
+from fbf.contrib.natural.date import duration, delta
 
 ## tinyurl import
 
@@ -63,6 +65,7 @@ import datetime
 import hashlib
 import copy
 import re
+import random
 
 ## exceptions
 
@@ -83,6 +86,8 @@ class RssNoSuchItem(RssException):
 
 ## defines
 
+starttime = None
+
 feedparser = getfeedparser()
 json = getjson()
 
@@ -96,7 +101,7 @@ possiblemarkup = {'separator': 'set this to desired item separator', \
 'tinyurl': "set this to 1 when you want to use tinyurls", 'skipmerge': \
 "set this to 1 if you want to skip merge commits", 'reverse-order': \
 'set this to 1 if you want the rss items displayed with oldest item first', \
-'nofeedname': "if you don't want the feedname shown"}
+'nofeedname': "if you don't want the feedname shown", 'hashtag': "hashtag to add add end of result"}
 
 ## global data
 
@@ -106,6 +111,9 @@ if not lastpoll.data: lastpoll.data = LazyDict() ; lastpoll.save()
 sleeptime = PlugPersist('sleeptime')
 if not sleeptime.data: sleeptime.data = LazyDict() ; sleeptime.save()
 
+worksleeptime = PlugPersist('worksleeptime')
+if not worksleeptime.data: worksleeptime.data = LazyDict() ; worksleeptime.save()
+
 runners = PlugPersist('runners')
 if not runners.data: runners.data = LazyDict() ; runners.save()
 
@@ -143,7 +151,7 @@ class Feed(Persist):
 
     """ item that contains rss data """
 
-    def __init__(self, name="nonameset", url="", owner="noownerset", itemslist=['title', 'link'], watchchannels=[], \
+    def __init__(self, name="nonameset", url="", owner="noownerset", itemslist=['title', 'link', 'author'], watchchannels=[], \
 sleeptime=15*60, running=0):
         if name:
             filebase = getdatadir() + os.sep + 'plugs' + os.sep + 'fbf.plugs.extra.rss' + os.sep + name
@@ -151,9 +159,9 @@ sleeptime=15*60, running=0):
             if not self.data: self.data = {}
             self.data = LazyDict(self.data)
             self.data.length = 200
-            self.data['name'] = self.data.name or name
-            self.data['url'] = self.data.url or url
-            self.data['owner'] = self.data.owner or owner
+            self.data['name'] = self.data.name or str(name)
+            self.data['url'] = self.data.url or str(url)
+            self.data['owner'] = self.data.owner or str(owner)
             self.data['result'] = []
             self.data['seen'] = self.data.seen or []
             self.data['watchchannels'] = self.data.watchchannels or list(watchchannels)
@@ -165,18 +173,18 @@ sleeptime=15*60, running=0):
     def checkseen(self, data, itemslist=["title", "link"]):
         d = {}
         for item in itemslist:
-            i = item # str(item, "utf-8")
-            try: d[i] = data[i]
+            try: d[item] = data[item]
             except (KeyError, TypeError): continue
-        digest = hashlib.md5(bytes(str(d), "utf-8")).hexdigest()
+        digest = hashlib.md5(str(d)).hexdigest()
         return digest in self.data.seen
 
     def setseen(self, data, itemslist=['title', 'link'], length=200):
         d = {}
         got = False
         for item in itemslist:
-            d[item] = data[item]
-        digest = hashlib.md5(bytes(str(d), "utf-8")).hexdigest()
+            try: d[item] = data[item]
+            except KeyError: continue
+        digest = hashlib.md5(str(d, "utf-8")).hexdigest()
         if digest not in self.data.seen: self.data.seen.insert(0, digest) ; got = True
         self.data.seen = self.data.seen[:self.data.length]
         return got
@@ -215,7 +223,7 @@ sleeptime=15*60, running=0):
         if name and name in etags.data: etag = etags.data[name]
         else: etag = None
         if data:
-            result = feedparser.parse(str(data.content, "utf-8"), etag=etag)
+            result = feedparser.parse(data.content, etag=etag)
             try: status = data.status_code
             except AttributeError: status = None
         else:
@@ -226,7 +234,6 @@ sleeptime=15*60, running=0):
             except AttributeError: status = None
         logging.info("status returned of %s feed is %s" % (name, status))
         if status == 304: return []
-        result = LazyDict(result)
         if result: set(self.data.url, result.entries, namespace='rss')
         if data:
             try: etag = etags.data[name] = data.headers.get('etag') ; logging.info("etag of %s set to %s" % (name, etags.data[name])) ; etags.sync()
@@ -245,10 +252,10 @@ sleeptime=15*60, running=0):
         """ refresh cached data of a feed. """
         if not self.data.running:
             logging.info("%s not enabled .. %s not syncing " % (self.data.name,strippassword( self.data.url)))
-            return False
+            return []
         logging.info("syncing %s - %s" % (self.data.name, strippassword(self.data.url)))
         result = self.fetchdata()
-        logging.info("%s entries fetched" % len(result))
+        logging.info("%s - %s entries fetched" % (self.data.name, len(result)))
         return result or []
 
     def updateseen(self, entries=None):
@@ -279,6 +286,7 @@ sleeptime=15*60, running=0):
         try:
             loopover = self.data.watchchannels
             logging.info("loopover in %s deliver is: %s" % (self.data.name, loopover))
+            nritems = 0
             for item in loopover:
                 logging.info("item is: %s" % str(item))
                 if not item: continue
@@ -299,42 +307,50 @@ sleeptime=15*60, running=0):
                      nick = getnick(bot, channel)
                      if not nick: logging.error("can't get nick to send to %s" % channel) ; continue
                 else: nick = None
+                if self.markup.get(jsonstring([name, type, channel]), 'withnotice'): how = "notice"
+                else: how = "msg"
                 if self.markup.get(jsonstring([name, type, channel]), 'reverse-order'): res2 = res2[::-1]
+                hashtag = self.markup.get(jsonstring([name, type, channel]), 'hashtag')
                 if self.markup.get(jsonstring([name, type, channel]), 'all-lines'):
                     for i in res2: 
                         response = self.makeresponse(name, type, [i, ], channel)
-                        try: bot.say(nick or channel, response, **{'headlines': True})
+                        if hashtag: response += " %s" % hashtag
+                        try: bot.say(nick or channel, response, how=how, speed=-1, **{'headlines': True}) ; nritems += 1
                         except Exception as ex: handle_exception()
                 else:
                     sep =  self.markup.get(jsonstring([name, type, channel]), 'separator')
                     if sep: response = self.makeresponse(name, type, res2, channel, sep=sep)
                     else: response = self.makeresponse(name, type, res2, channel)
-                    try: bot.say(nick or channel, response, **{'headlines': True})
+                    if hashtag: response += " %s" % hashtag
+                    try: bot.say(nick or channel, response, how=how, speed=-1, **{'headlines': True}) ; nritems += 1
                     except Exception as ex: handle_exception()
-            return True
-        except Exception as ex: handle_exception(txt=name) ; return False
+            logging.warn("DELIVERED %s %s items on: %s" % (nritems, name, str(loopover)))
+            return nritems
+        except Exception as ex: handle_exception(txt=name) ; return str(ex)
 
     def makeresponse(self, name, type, res, channel, sep=" || "):
         """ loop over result to make a response. """
+        if not sep: sep = ""
         if self.markup.get(jsonstring([name, type, channel]), 'nofeedname'): result = ""
         else: result = "<b>[%s]</b> - " % name 
         try: itemslist = self.itemslists.data[jsonstring([name, type, channel])]
         except KeyError:
-            itemslist = self.itemslists.data[jsonstring([name, type, channel])] = ['title', 'link']
+            itemslist = self.itemslists.data[jsonstring([name, type, channel])] = ['title', 'link', "author"]
             self.itemslists.save()
         for j in res:
             if self.markup.get(jsonstring([name, type, channel]), 'skipmerge') and 'Merge branch' in j['title']: continue
             resultstr = ""
             for i in itemslist:
-                try:
+                try: 
                     item = getattr(j, i)
                     if not item: continue
-                    item = str(item)
+                    try: item = str(item)
+                    except: pass
                     if item.startswith('http://'):
                         if self.markup.get(jsonstring([name, type, channel]), 'tinyurl'):
                             try:
                                 tinyurl = get_tinyurl(item)
-                                logging.warn('tinyurl is: %s' % str(tinyurl))
+                                logging.debug(' tinyurl is: %s' % str(tinyurl))
                                 if not tinyurl: resultstr += "%s - " % item
                                 else: resultstr += "%s - " % tinyurl[0]
                             except Exception as ex:
@@ -342,7 +358,7 @@ sleeptime=15*60, running=0):
                                 resultstr += "%s - " % item
                         else: resultstr += "%s - " % item
                     else: resultstr += "%s - " % item.strip()
-                except (KeyError, AttributeError, TypeError) as ex: logging.warn('%s - %s' % (name, str(ex))) ; continue
+                except (UnicodeDecodeError, KeyError, AttributeError, TypeError) as ex: logging.warn('%s - %s' % (name, str(ex))) ; continue
             resultstr = resultstr[:-3]
             if resultstr: result += "%s %s " % (resultstr, sep)
         return result[:-(len(sep)+2)]
@@ -396,9 +412,10 @@ class Rssdict(PlugPersist):
 
     def add(self, name, url, owner):
         """ add rss item. """
+        if "<" in name or ">" in name: logging.error("not a proper feed name %s" % name) ; return
         logging.warn('adding %s - %s - (%s)' % (name, strippassword(url), owner))
         if name not in self.data['names']: self.data['names'].append(name)
-        self.feeds[name] = Feed(name, url, owner, ['title', 'link'])
+        self.feeds[name] = Feed(name, url, owner, ['title', 'link', "author"])
         self.data['urls'][url] = name
         self.feeds[name].save()
         self.watch(name)
@@ -446,6 +463,7 @@ class Rssdict(PlugPersist):
     def watch(self, name, sleepsec=300):
         """ start a watcher thread """
         logging.warn('starting %s rss feed watcher' % name)
+        if "<" in name or ">" in name: logging.error("not a proper feed name %s" % name) ; return
         rssitem = self.byname(name)
         if rssitem == None: raise RssNoItem()
         rssitem.data.running = 1
@@ -454,7 +472,9 @@ class Rssdict(PlugPersist):
         rssitem.save()
         if not name in runners.data: runners.data[name] = "bla" ; runners.save()
         sleeptime.data[name] = sleepsec
+        worksleeptime.data[name] = sleepsec
         sleeptime.save()
+        worksleeptime.save()
         logging.warn('%s started' % name)
 
 ## Rsswatcher class
@@ -518,15 +538,30 @@ class Rsswatcher(Rssdict):
     def syncdeliver(self, name):
         """ sync a feed. """
         feed = self.byname(name)
+        result = []
+        delivered = 0
+        checked = 0
         if feed:
             result = feed.sync()
             if result:
                 res2 = feed.check(result)
-                if res2: feed.deliver(res2)
-                else: logging.debug("no result from check - %s" % name)
-            else: logging.debug("not result from sync - %s" % name)
-        else: logging.warn("can't find %s feed" % name)
-        
+                if res2:
+                    checked = len(res2)
+                    delivered = feed.deliver(res2)
+                    sl = worksleeptime.data[name]
+                    if sl > sleeptime.data[name]: worksleeptime.data[name] = sleeptime.data[name]
+                else:
+                    logging.info("%s - nothing to deliver" % name)
+                    sl = worksleeptime.data[name] 
+                    if sl < sleeptime.data[name]: worksleeptime.data[name] += (sl*0.1 + random.randint(1,10))
+            else: logging.info("%s - no result from sync" % name)
+        else: logging.warn("can't find %s feed" % name) ; return "no feed"
+        if delivered: ret = "delivered: %s" % delivered
+        elif checked: ret = "checked: %s" % checked
+        elif len(result): ret = "fetched: %s" % len(result)
+        else: ret = "none done"
+        return ret
+       
     def ownercheck(self, name, userhost):
         """ check if userhost is the owner of feed. """
         try:
@@ -539,6 +574,8 @@ class Rsswatcher(Rssdict):
         """ not implemented yet. """
         sleeptime.data[name] = interval
         sleeptime.save()
+        worksleeptime.data[name] = interval
+        worksleeptime.save()
 
     def stopwatchers(self):
         """ stop all watcher threads. """
@@ -546,7 +583,7 @@ class Rsswatcher(Rssdict):
             if z.data.running: z.data.stoprunning = 1 ; z.data.running = 0
             z.save()
 
-    def dowatch(self, name, sleeptime=1800):
+    def dowatch(self, name, sleeptime=300):
         """ start a watcher. """
         rssitem = self.byname(name)
         if not rssitem == None:
@@ -639,7 +676,7 @@ class Rsswatcher(Rssdict):
         res = []
         for name in self.data['names']:
             feed = self.byname(name)
-            if feed: res.append(str(feed.search(item, search), "utf-8"))
+            if feed: res.append(str(feed.search(item, search)))
         return res
 
     def all(self, name, item):
@@ -666,7 +703,7 @@ class Rsswatcher(Rssdict):
         target = channel
         if not jsonstring([botname, bottype, target]) in rssitem.data.watchchannels and not [botname, bottype, target] in rssitem.data.watchchannels:
             rssitem.data.watchchannels.append([botname, bottype, target])
-        rssitem.itemslists[jsonstring([name, bottype, target])] = ['title', 'link']
+        rssitem.itemslists[jsonstring([name, bottype, target])] = ['title', 'link', "author"]
         rssitem.markup.set(jsonstring([name, bottype, target]), 'tinyurl', 1)
         rssitem.data.running = 1
         rssitem.data.stoprunning = 0
@@ -707,7 +744,6 @@ assert(watcher)
 def size():
     return "%s (feeds)" % watcher.size()
 
-
 ## dosync function
 
 def dummycb(bot, event): pass
@@ -719,8 +755,8 @@ def dosync(feedname):
     try:
         logging.info("doing sync of %s" % feedname)
         global watcher 
-        watcher.syncdeliver(feedname)
-    except RssException as ex: logging.error("%s - error: %s" % (feedname, str(ex)))
+        return watcher.syncdeliver(feedname)
+    except RssException as ex: logging.error("%s - error: %s" % (feedname, str(ex))) ; return str(ex)
 
 ## shouldpoll function
 
@@ -729,30 +765,31 @@ def shouldpoll(name, curtime):
     global lastpoll
     try: lp = lastpoll.data[name]
     except KeyError: lp = lastpoll.data[name] = time.time() ; lastpoll.sync()
-    global sleeptime
-    try: st = sleeptime.data[name]
-    except KeyError: st = sleeptime.data[name] = 900 ; sleeptime.sync()
+    global worksleeptime
+    try: st = worksleeptime.data[name]
+    except KeyError: st = worksleeptime.data[name] = 900 ; worksleeptime.sync()
     logging.debug("pollcheck - %s - %s - remaining %s" % (name, time.ctime(lp), (lp + st) - curtime))
     if curtime - lp > st: return True
 
+## dodata function
+
+def dodata(data, name):
+    watcher.handle_data(data, name=name)    
+
 ## doperiodical function
 
 def doperiodical(*args, **kwargs):
     """ rss periodical function. """
     got = False
     curtime = time.time()
+    #sleeptime.save()
     for feed in watcher.data.names:
         if not watcher.shouldpoll(feed, curtime): continue
         lastpoll.data[feed] = curtime
         got = True
-        logging.debug("periodical - launching %s" % feed)
-        try:
-            from google.appengine.ext.deferred import defer
-            defer(dosync, feed)
-        except ImportError:
-            try: start_new_thread(dosync, (feed, ))
-            except Exception as ex: handle_exception() ; return
-        time.sleep(2)
+        logging.info("launching %s" % feed)
+        threadrunner.put(8, feed, dosync, (feed, ))
+        time.sleep(1)
     if got: lastpoll.save()
 
 callbacks.add('TICK60', doperiodical)
@@ -762,7 +799,9 @@ callbacks.add('TICK60', doperiodical)
 def init():
     """ initialize the rss plugin. """
     if not runners.data: start_new_thread(watcher.checkrunners, ())
-    
+    global starttime
+    starttime = time.time()
+ 
 ## shutdown function
 
 def shutdown():
@@ -781,12 +820,62 @@ def save():
     """ save watcher data. """
     watcher.save()
 
+## rss-lastpoll command
+
+def handle_rsslastpoll(bot, event):
+    if not event.rest:
+        result = {}
+        for feedname, last in lastpoll.data.items():
+            if last - starttime < 0: last = starttime
+            try: result[feedname] = duration(last, plain=True)
+            except Exception as ex: logging.warn("error determining lastpoll of %s: %s" % (feedname, str(ex))) ; continue
+        event.reply("lastpolled: ", result)
+        return
+    name = event.args[0]
+    try:
+        last = lastpoll.data[name]
+        if last - starttime < 0: last = starttime
+        event.reply("last poll of the %s feed: %s" % (name, duration(last, plain=True))) 
+    except KeyError: event.reply("we don't run a %s feed" % name)
+    except Exception as ex: event.reply("error occured: %s" % str(ex))
+
+cmnds.add("rss-lastpoll", handle_rsslastpoll, ["OPER", "USER"])
+examples.add("rss-lastpoll", "shows time when a feed was last polled", "1) rss-lastpoll fbfbot 2) rss-lastpoll all")
+
+## rss-nextpoll command
+
+def handle_rssnextpoll(bot, event):
+    if not event.rest:
+        result = {}
+        for name, last in lastpoll.data.items():
+            try: sleep = worksleeptime.data[name]
+            except KeyError:
+                try: sleep = sleeptime.data[name]
+                except KeyError: continue
+            if last - starttime < 0: last = starttime
+            try: result[name] = duration(sleep + last, precision=0.1, plain=True)
+            except Exception as ex: logging.warn("error determining lastpoll of %s: %s" % (name, str(ex))) ; continue
+        event.reply("lastpolled: ", result)
+        return
+    name = event.args[0]
+    try:
+        sleep = worksleeptime.data[name]
+        last = lastpoll.data[name]
+        if last - starttime < 0: last = starttime
+        event.reply("time to next poll of the %s feed: %s" % (name, duration(last + sleep, precision=0.1, plain=True)))
+    except KeyError: event.reply("we don't run a %s feed" % name)
+    except Exception as ex: event.reply("error occured: %s" % str(ex))
+
+cmnds.add("rss-nextpoll", handle_rssnextpoll, ["OPER", "USER"])
+examples.add("rss-nextpoll", "shows time until a feed will be polled", "1) rss-nextpoll fbfbot 2) rss-nextpoll all")
+
 ## rss-add command
 
 def handle_rssadd(bot, ievent):
     """ arguments: <feedname> <url> - add a rss item. """
     try: (name, url) = ievent.args
     except ValueError: ievent.missing('<feedname> <url>') ; return
+    if "<" in name or ">" in name: ievent.reply("not a proper feed name %s" % name) ; return
     if watcher.checkfeed(url, ievent):
         feed = watcher.add(name, url, ievent.userhost)
         feed.updateseen()
@@ -794,7 +883,7 @@ def handle_rssadd(bot, ievent):
     else: ievent.reply('%s is not valid' % strippassword(url))
 
 cmnds.add('rss-add', handle_rssadd, 'USER')
-examples.add('rss-add', 'add a feed to the rsswatcher', 'rss-add fbfbot http://code.google.com/feeds/p/fbfbot/hgchanges/basic')
+examples.add('rss-add', 'add a feed to the rsswatcher', 'rss-add fbfbot https://github.com/feedbackflow/fbfbot/commits/master.atom')
 
 ## rss-register command
 
@@ -805,6 +894,7 @@ def handle_rssregister(bot, ievent):
     if watcher.byname(name):
         ievent.reply('we already have a feed with %s name .. plz choose a different name' % name)
         return
+    if "<" in name or ">" in name: ievent.reply("not a proper feed name %s" % name) ; return
     if watcher.checkfeed(url, ievent):
         feed = watcher.add(name, url, ievent.userhost)
         feed.updateseen()
@@ -814,7 +904,7 @@ def handle_rssregister(bot, ievent):
     else: ievent.reply('%s is not valid' % strippassword(url))
 
 cmnds.add('rss-register', handle_rssregister, 'USER')
-examples.add('rss-register', 'register and start a rss feed', 'rss-register fbfbot-hg http://code.google.com/feeds/p/fbfbot/hgchanges/basic')
+examples.add('rss-register', 'register and start a rss feed', 'rss-register fbfbot https://github.com/feedbackflow/fbfbot/commits/master.atom')
 
 ## rss-del command
 
@@ -850,7 +940,7 @@ def handle_rsswatch(bot, ievent):
     """ arguments: <feedname> [secondstosleep] - start watching a feed. """
     try: name, sleepsec = ievent.args
     except ValueError:
-        try: name = ievent.args[0] ; sleepsec = 1800
+        try: name = ievent.args[0] ; sleepsec = 300
         except IndexError: ievent.missing('<feedname> [secondstosleep]') ; return
     try: sleepsec = int(sleepsec)
     except ValueError: ievent.reply("time to sleep needs to be in seconds") ; return
@@ -861,6 +951,7 @@ def handle_rsswatch(bot, ievent):
         rssitem = watcher.byname(feed)
         if rssitem == None: continue
         if name not in sleeptime.data: sleeptime.data[feed] = sleepsec ; sleeptime.save()
+        if name not in worksleeptime.data: worksleeptime.data[feed] = sleepsec ; worksleeptime.save()
         try: watcher.watch(feed, sleepsec)
         except Exception as ex: ievent.reply('%s - %s' % (feed, str(ex))) ; continue
         got.append(feed)
@@ -1001,12 +1092,21 @@ def handle_rssadditem(bot, ievent):
     except ValueError: ievent.missing('<feedname> <token>') ; return
     if ievent.options and ievent.options.channel: target = ievent.options.channel
     else: target = ievent.channel
-    feed = watcher.byname(name)
-    if not feed: ievent.reply("we don't have a %s feed" % name) ; return
-    try: feed.itemslists.data[jsonstring([name, bot.type, target])].append(item)
-    except KeyError: feed.itemslists.data[jsonstring([name, bot.type, target])] = ['title', 'link']
-    feed.itemslists.save()
-    ievent.reply('%s added to (%s,%s) itemslist' % (item, name, target))
+    if name == "all": todo = watcher.list()
+    else: todo = [name, ]
+    errors = {}
+    done = []
+    for name in todo:
+        try:
+            feed = watcher.byname(name)
+            if not feed: ievent.reply("we don't have a %s feed" % name) ; return
+            try: feed.itemslists.data[jsonstring([name, bot.type, target])].append(item)
+            except KeyError: feed.itemslists.data[jsonstring([name, bot.type, target])] = ['title', 'link', "author"]
+            feed.itemslists.save()
+        except Exception as ex: errors[name] = str(ex) ; continue
+        done.append(name)
+    if errors: event.reply("errors occured: ", errors)
+    ievent.reply('%s added to: ' % item, done)
 
 cmnds.add('rss-additem', handle_rssadditem, ['RSS', 'USER'])
 examples.add('rss-additem', 'add a token to the itemslist (per user/channel)', 'rss-additem fbfbot link')
@@ -1019,13 +1119,22 @@ def handle_rssdelitem(bot, ievent):
     except ValueError: ievent.missing('<feedname> <token>') ; return
     if ievent.options and ievent.options.channel: target = ievent.options.channel
     else: target = ievent.channel
-    rssitem =  watcher.byname(name)
-    if not rssitem: ievent.reply("we don't have a %s feed" % name) ; return
-    try:
-        rssitem.itemslists.data[jsonstring([name, bot.type, target])].remove(item)
-        rssitem.itemslists.save()
-    except (ValueError, KeyError): ievent.reply("we don't have a %s rss feed" % name) ; return
-    ievent.reply('%s removed from (%s,%s) itemslist' % (item, name, target))
+    if name == "all": todo = watcher.list()
+    else: todo = [name, ]
+    errors = {}
+    done = []
+    for name in todo:
+        try:
+            rssitem =  watcher.byname(name)
+            if not rssitem: errors[name] = "no such feed" ; continue
+            try:
+                rssitem.itemslists.data[jsonstring([name, bot.type, target])].remove(item)
+                rssitem.itemslists.save()
+            except (ValueError, KeyError) as ex: errors[name] = str(ex) ; continue
+        except Exception as ex: errors[name] = str(ex) ; continue
+        done.append(name)
+    if errors: ievent.reply("errors occured: ", errors)
+    ievent.reply('%s removed from: ' % item, done)
 
 cmnds.add('rss-delitem', handle_rssdelitem, ['RSS', 'USER'])
 examples.add('rss-delitem', 'remove a token from the itemslist (per user/channel)', 'rss-delitem fbfbot link')
@@ -1045,12 +1154,22 @@ def handle_rssmarkup(bot, ievent):
     """ arguments: <feedname> - show the markup of a feed. """
     try: name = ievent.args[0]
     except IndexError: ievent.missing('<feedname>') ; return
-    rssitem =  watcher.byname(name)
-    if not rssitem: ievent.reply("we don't have a %s feed" % name) ; return
-    if ievent.options and ievent.options.channel: target = ievent.options.channel
-    else: target = ievent.channel
-    try: ievent.reply(str(rssitem.markup[jsonstring([name, bot.type, target])]))
-    except KeyError: pass
+    if name == "all": todo = watcher.list()
+    else: todo = [name, ]
+    errors = {}
+    done = []
+    for name in todo:
+        try:
+            rssitem =  watcher.byname(name)
+            if not rssitem: ievent.reply("we don't have a %s feed" % name) ; return
+            if ievent.options and ievent.options.channel: target = ievent.options.channel
+            else: target = ievent.channel
+            try: ievent.reply(str(rssitem.markup[jsonstring([name, bot.type, target])]))
+            except KeyError: pass
+        except Exception as ex: errors[name] = str(ex) ; continue
+        done.append(name)
+    if errors: event.reply("errors occured: ", errors)
+    ievent.reply('added to: ', done)
 
 cmnds.add('rss-markup', handle_rssmarkup, ['RSS', 'USER'])
 examples.add('rss-markup', 'show markup list for a feed (per user/channel)', 'rss-markup fbfbot')
@@ -1061,17 +1180,26 @@ def handle_rssaddmarkup(bot, ievent):
     """ arguments: <feedname> <item> <value> - add a markup to a feeds markuplist. """
     try: (name, item, value) = ievent.args
     except ValueError: ievent.missing('<feedname> <item> <value>') ; return
-    rssitem =  watcher.byname(name)
-    if not rssitem: ievent.reply("we don't have a %s feed" % name) ; return
-    if ievent.options and ievent.options.channel: target = ievent.options.channel
-    else: target = ievent.channel
-    try: value = int(value)
-    except ValueError: pass
-    try:
-        rssitem.markup.set(jsonstring([name, bot.type, target]), item, value)
-        rssitem.markup.save()
-        ievent.reply('%s added to (%s,%s) markuplist' % (item, name, target))
-    except KeyError: ievent.reply("no (%s,%s) feed available" % (name, target))
+    if name == "all": todo = watcher.list()
+    else: todo = [name, ]
+    errors = {}
+    done = []
+    for name in todo:
+        try:
+            rssitem =  watcher.byname(name)
+            if not rssitem: continue
+            if ievent.options and ievent.options.channel: target = ievent.options.channel
+            else: target = ievent.channel
+            try: value = int(value)
+            except ValueError: pass
+            try:
+                rssitem.markup.set(jsonstring([name, bot.type, target]), item, value)
+                rssitem.markup.save()
+            except KeyError: ievent.reply("no (%s,%s) feed available" % (name, target))
+        except Exception as ex: errors[name] = str(ex) ; continue
+        done.append(name)
+    if errors: event.reply("errors occured: ", errors)
+    ievent.reply('%s added to: ' % item, done)
 
 cmnds.add('rss-addmarkup', handle_rssaddmarkup, ['RSS', 'USER'])
 examples.add('rss-addmarkup', 'add a markup option to the markuplist (per user/channel)', 'rss-addmarkup fbfbot all-lines 1')
@@ -1082,14 +1210,23 @@ def handle_rssdelmarkup(bot, ievent):
     """ arguments: <feedname> <item> - delete markup from a feeds markuplist. """
     try: (name, item) = ievent.args
     except ValueError: ievent.missing('<feedname> <item>') ; return
-    rssitem =  watcher.byname(name)
-    if not rssitem: ievent.reply("we don't have a %s feed" % name) ; return
-    if ievent.options and ievent.options.channel: target = ievent.options.channel
-    else: target = ievent.channel
-    try: del rssitem.markup[jsonstring([name, bot.type, target])][item]
-    except (KeyError, TypeError): ievent.reply("can't remove %s from %s feed's markup" %  (item, name)) ; return
-    rssitem.markup.save()
-    ievent.reply('%s removed from (%s,%s) markuplist' % (item, name, target))
+    if name == "all": todo = watcher.list()
+    else: todo = [name, ]
+    errors = {}
+    done = []
+    for name in todo:
+        try:
+            rssitem =  watcher.byname(name)
+            if not rssitem: continue
+            if ievent.options and ievent.options.channel: target = ievent.options.channel
+            else: target = ievent.channel
+            try: del rssitem.markup[jsonstring([name, bot.type, target])][item]
+            except (KeyError, TypeError): ievent.reply("can't remove %s from %s feed's markup" %  (item, name)) ; return
+            rssitem.markup.save()
+        except Exception as ex: errors[name] = str(ex) ; continue
+        done.append(name)
+    if errors: event.reply("errors occured: ", errors)
+    ievent.reply('%s added to: ' % item, done)
 
 cmnds.add('rss-delmarkup', handle_rssdelmarkup, ['RSS', 'USER'])
 examples.add('rss-delmarkup', 'remove a markup option from the markuplist (per user/channel)', 'rss-delmarkup fbfbot all-lines')
@@ -1146,7 +1283,7 @@ def handle_rsssleeptime(bot, ievent):
     """ arguments: <feedname> - get sleeptime of rss item. """
     try: name = ievent.args[0]
     except IndexError: ievent.missing('<feedname>') ; return
-    try: ievent.reply('sleeptime for %s is %s seconds' % (name, str(sleeptime.data[name])))
+    try: ievent.reply('sleeptime for %s is %s (%s) seconds' % (name, str(sleeptime.data[name]), str(worksleeptime.data[name])))
     except KeyError: ievent.reply("can't get sleeptime for %s" % name)
 
 cmnds.add('rss-sleeptime', handle_rsssleeptime, 'USER')
@@ -1180,7 +1317,7 @@ def handle_rssget(bot, ievent):
     rssitem = watcher.byname(name)
     if rssitem == None: ievent.reply("we don't have a %s rss item" % name) ; return
     try: result = watcher.getdata(name)
-    except Exception as ex: handle_exception() ; ievent.reply('%s error: %s' % (name, str(ex))) ; return
+    except Exception as ex: ievent.reply('%s error: %s' % (name, str(ex))) ; return
     if rssitem.markup.get(jsonstring([name, bot.type, channel]), 'reverse-order'): result = result[::-1]
     response = rssitem.makeresponse(name, bot.type, result, ievent.channel)
     if response: ievent.reply("results of %s: %s" % (name, response))
@@ -1244,7 +1381,7 @@ def handle_rssseturl(bot, ievent):
     else: ievent.reply('failed to set url of %s to %s' % (name, strippassword(url)))
 
 cmnds.add('rss-seturl', handle_rssseturl, ['OPER', ])
-examples.add('rss-seturl', 'set the url of a feed', 'rss-seturl fbfbot-hg http://code.google.com/feeds/p/fbfbot/hgchanges/basic')
+examples.add('rss-seturl', 'set the url of a feed', 'rss-seturl fbfbot https://github.com/feedbackflow/fbfbot/commits/master.atom')
 
 ## rss-itemslist
 
@@ -1368,6 +1505,7 @@ def handle_rssimport(bot, ievent):
             assert(name)
             assert(url)
             logging.warn("import - adding %s - %s" % (name, strippassword(url)))
+            if "<" in name or ">" in name: continue
             watcher.add(fromenc(name), fromenc(url), ievent.userhost)
             teller += 1
         except Exception as ex:
@@ -1380,4 +1518,4 @@ def handle_rssimport(bot, ievent):
         ievent.reply("there were errors: ", errlist)
 
 cmnds.add('rss-import', handle_rssimport, ['OPER', ])
-examples.add('rss-import', 'import rss feeds from a remote OPML file.', 'rss-import http:///feeds.opml')
+examples.add('rss-import', 'import rss feeds from a remote OPML file.', 'rss-import http://feedbackflow.org/feeds.opml')
